#include <stdlib.h>
#include <stdio.h>
void MvdSetAmuBits( char cline[80], int amubit[8][33] )
{
   /* scan a command line to look for definitions of AMU bits           */

   /* input cline, hmbit                                                */

   /* command  bits used        purpose                                 */
   /* _______  _________        __________                              */
   /* amumux   amubit[i][0]     Turn on a single AMU spy channel output */
   /*          amubit[i][25]    This means setting amubit[i][0] and     */
   /*          amubit[i][1-5]   amubit[i][25] to 1 to enable output for */
   /*                           the selected channel or to 0 to turn it */
   /*                           off. The next arguement is the channel  */
   /*                           number (from 0 to 31) which is used to  */
   /*                           set bits 1-5.                           */
   /*                           Note, only 1 AMU spy channel should be  */
   /*                           on at one time, so this command turns   */
   /*                           all other channels off.                 */
   /* --       amubit[i][6]     correlator enable bit. This should have */
   /*                           a value opposite to the heap manager's  */
   /*                           raw mode bit and therefore it is set via*/
   /*                           the heap manager command -- not here    */
   /* vref     amubit[i][7-12]  Set vref DAC. This DAC uses 6 bits.     */
   /*                           It ranges from 0.2Volts to 4.8Volts.    */
   /*                           dac = ((V-0.2)/(4.8-0.2))*63 + 0.5      */
   /*                           amubit[i][12]=MSB, amubit[i][7]=LSB     */
   /* vcorr    amubit[i][13-18] Set vcorr DAC. This DAC uses 6 bits.    */
   /*                           It ranges from 0.1Volts to 2.5Volts.    */
   /*                           dac = ((V-0.1)/(2.5-0.1))*63 + 0.5      */
   /*                           amubit[i][18]=MSB, amubit[i][13]=LSB    */
   /* iref     amubit[i][19-24] Set iref  DAC. This DAC uses 6 bits.    */
   /*                           It ranges from 2 to 70 microamps        */
   /*                           dac = ((70-V)/(70-2))*63 + 0.5          */
   /*                           amubit[i][24]=MSB, amubit[i][19]=LSB    */

   /* Author = JPSullivan 2 and 3-Aug-2000 (based on labwindows code    */
   /*          originally written by BSchlei and SSRyu)                 */

   /* **********************************************************        */

   void MvdSetAmuMux (  char cline[80], int amuchan, int amubit[8][33] );
   void MvdSetTgvMux (  char cline[80], int amuchan, int amubit[8][33] );
   void MvdSetAmuVref(  char cline[80], int amuchan, int amubit[8][33] );
   void MvdSetAmuVcorr( char cline[80], int amuchan, int amubit[8][33] );
   void MvdSetAmuViref( char cline[80], int amuchan, int amubit[8][33] );

   int i;
   int ichan;
   char arg1[80];
   char arg2[80];
   char Cmd_amumux[7]={'a','m','u','m','u','x'};
   char Cmd_tgvmux[7]={'t','g','v','m','u','x'};
   char Cmd_vref[5]={'v','r','e','f'};
   char Cmd_vcorr[6]={'v','c','o','r','r'};
   char Cmd_iref[5]={'i','r','e','f'};

   /* ********************************************************** */

   sscanf (cline, "%s%s%d", arg1,arg2,&ichan );
   if ( ichan>8 )
   {
      fprintf ( stdout, 
                "MvdSetAmuBits: AMU channel (=%d)  must less than 8, return without doing anything\n",ichan);
      return;
   }
   if ( ichan<0 )
   {
      fprintf ( stdout,
                "MvdSetAmuBits: AMU channel (=%d) number must postive, return without doing anything\n",ichan);
      return;
   }

   /* _____________________________________________________ */

   for ( i=0; i<strlen(arg1); i++ )
      arg1[i] = tolower(arg1[i]);


   if ( arg1[0] != 'a' || arg1[1] != 'm' ) 
      return;
   
   /* _____________________________________________________ */

   if ( strcmp ( Cmd_amumux, arg2 ) == 0 ) 
   {
     MvdSetAmuMux ( cline, ichan, amubit );
   }
   else if ( strcmp ( Cmd_tgvmux, arg2 ) == 0 ) 
   {
     MvdSetTgvMux ( cline, ichan, amubit );
   }
   else if ( strcmp ( Cmd_vref, arg2 ) == 0 ) 
   {
     MvdSetAmuVref ( cline, ichan, amubit );
   }
   else if ( strcmp ( Cmd_vcorr, arg2 ) == 0 ) 
   {
     MvdSetAmuVcorr ( cline, ichan, amubit );
   }
   else if ( strcmp ( Cmd_iref, arg2 ) == 0 ) 
   {
     MvdSetAmuIref ( cline, ichan, amubit );
   }

   return;
}
