#include <stdlib.h>
#include <stdio.h>
void MvdSetTgvBits( char cline[80], int tgvbit[8][190] )
{
   /* scan a command line to look for definitions of TGV bits           */

   /* input cline, hmbit                                                */

   /* command  bits used          purpose                               */
   /* _______  _________          __________                            */
   /* csdis    tgvbit[i][1-32]    current sum (aka discriminator)       */
   /*                             disable, 1 means disable              */
   /*                             command syntax is:                    */
   /*                             tgv csdis chip chan onoff             */
   /*                             chip = 0 means all 8 chips on the MCM,*/
   /*                                  = 1-8 refers to single chips     */
   /*                             chan = 0 means all channels on a chip,*/
   /*                                  = 1-32 means a single channel    */
   /*                             onoff=0 is on (normal), 1 is disable  */
   /* dtwk     tgvbit[i][32-95]   Discriminator tweak bits.             */
   /*                             tgv dtwk chip chan ithr               */
   /*                             Bits 63-32 = "bit a" for chan 0-31.   */
   /*                             Bits 95-64 = "bit b" for chan 0-31.   */
   /*                             The code assumes that "bit a" is the  */
   /*                             low order bit and "bit b" is the      */
   /*                             higher order bit in a two bit word    */
   /*                             whose value is ithr in the command.   */
   /*                             These bits are intended to provide a  */
   /*                             fine channel to channel adjust for the*/
   /*                             coarse threshold set for all 32       */
   /*                             channels on a chip via the Vthr DAC.  */
   /* predis   tgvbit[i][96-127]  Preamp disable , 1 means disable      */
   /*                             command syntax is:                    */
   /*                             tgv predis chip chan onoff            */
   /*                             chip = 0 means all 8 chips on the MCM,*/
   /*                                  = 1-8 refers to single chips     */
   /*                             chan = 0 means all channels on a chip,*/
   /*                                  = 1-32 means a single channel    */
   /*                             onoff=0 is on (normal), 1 is disable  */
   /* predisc  tgvbit[i][96-127]  Preamp disable, similar to predis, but*/
   /*                             count channels from 0-255 and leave   */
   /*                             chip number out of the command.       */
   /*                             command syntax is:                    */
   /*                             tgv predis chan onoff                 */
   /*                             chan = 0-255                          */
   /*                             onoff=0 is on (normal), 1 is disable  */
   /* calen    tgvbit[i][128-159] Bench cal eneable, 1 means on         */
   /*                             command syntax is:                    */
   /*                             tgv calen chip chan onoff             */
   /*                             chip = 0 means all 8 chips on the MCM,*/
   /*                                  = 1-8 refers to single chips     */
   /*                             chan = 0 means all channels on a chip,*/
   /*                                  = 1-32 means a single channel    */
   /*                             onoff=0 is off, =1 is on              */
   /* vthr     tgvbit[i][160-165] Set the VTHR  DAC -- sets threshold on*/
   /*                             discriminator. This adjustment's      */
   /*                             sensitivitiy is controlled by Vgate.  */
   /*                             A lower setting means higher threshold*/
   /*                             Vthr=0 is the highest threshold.      */
   /*                             1 mip seems to be around 1.4V         */
   /*                             It ranges from 0.00 to 1.86V          */
   /*                             dac = ((1.86-V)/(1.86-0))*63 + 0.5    */
   /*                             tgvbit[i][160]=MSB, tgvbit[i][165]=LSB*/
   /* vmid     tgvbit[i][166-171] Set the Vmid  DAC (I'm not real clear */
   /*                             about this DAC. It sets an offset V,  */
   /*                             but I forgot on what.)                */
   /*                             It ranges from 1.71 to 4.50V          */
   /*                             dac = ((V-1.71)/(4.50-1.71))*63 + 0.5 */
   /*                             tgvbit[i][166]=MSB, tgvbit[i][171]=LSB*/
   /* vgate    tgvbit[i][172-177] Set the Vgate DAC (I'm not real clear */
   /*                             about this DAC, but it is related to  */
   /*                             the discriminator performance -- I    */
   /*                             it controls the sensitivity of the    */
   /*                             threshold to the THRESH_DAC)          */
   /*                             It ranges from 0.006 to 3.68V         */
   /*                             dac = ((V-.006)/(3.68-.006))*63 + 0.5 */
   /*                             tgvbit[i][172]=MSB, tgvbit[i][177]=LSB*/
   /* vfb      tgvbit[i][178-183] Set the Vfb DAC (baseline restoration)*/
   /*                             It ranges from 0.006 to 5.02V         */
   /*                             Around 3.0V seems to work best, higher*/
   /*                             number means return to baseline slower*/
   /*                             dac = ((V-.006)/(5.02-.006))*63 + 0.5 */
   /*                             tgvbit[i][178]=MSB, tgvbit[i][183]=LSB*/
   /* vcal     tgvbit[i][184-189] Set the Vcal DAC (for bench cal pulse)*/
   /*                             It ranges from 3.48 to 5.0V           */
   /*                             3.48 corresponds to the biggest pulse */
   /*                             dac = ((V-3.48)/(5-3.48))*63 + 0.5    */
   /*                             tgvbit[i][184]=MSB, tgvbit[i][189]=LSB*/

   /* Author = JPSullivan 3 and 4-Aug-2000 (based on labwindows code    */
   /*          originally written by BSchlei and SSRyu)                 */
   /* 26-Jun-2001: JPS added predisc command                            */

   /* **********************************************************        */

   void MvdSetTgvVcal( char cline[80], int tgvchan, int tgvbit[8][190] );

   int i;
   int ichan;
   char arg1[80];
   char arg2[80];
   char Cmd_vcal[5]  = {'v','c','a','l'};
   char Cmd_vfb[4]   = {'v','f','b'};
   char Cmd_vgate[6] = {'v','g','a','t','e'};
   char Cmd_vmid[5]  = {'v','m','i','d'};
   char Cmd_vthr[5]  = {'v','t','h','r'};
   char Cmd_calen[6] = {'c','a','l','e','n'};
   char Cmd_predis[7]= {'p','r','e','d','i','s'};
   char Cmd_predisc[8]= {'p','r','e','d','i','s','c'};
   char Cmd_dtwk[5]  = {'d','t','w','k'};
   char Cmd_csdis[6] = {'c','s','d','i','s'};

   /* ********************************************************** */

   sscanf (cline, "%s%s%d", arg1,arg2,&ichan );
   if ( ichan>8 )
   {
      if ( strcmp ( Cmd_predisc, arg2 ) == 0 )
      {
         /* special case, command = predisc, which means this channel  */
         /* number counts from 0-255, so a number greater than 8 is OK */
         if ( ichan>255 )
         {
            fprintf ( stderr,
                 "MvdSetTgvBits: predisc channel (=%d) number must less than 255, return without doing anything\n",ichan);
            return;
         }
      }
      else
      {
        fprintf ( stderr,
                 "MvdSetTgvBits: TGV channel (=%d) number must less than 8, return without doing anything\n",ichan);
        return;
      }
   }
   if ( ichan<0 )
   {
      fprintf ( stderr, 
                "MvdSetTgvBits: TGV channel (=%d) number must postive, return without doing anything\n",ichan);
      return;
   }

   /* _____________________________________________________ */

   for ( i=0; i<strlen(arg1); i++ )
      arg1[i] = tolower(arg1[i]);

   if ( arg1[0] != 't' || arg1[1] != 'g' ) 
      return;
   
   /* _____________________________________________________ */

   if ( strcmp ( Cmd_vcal, arg2 ) == 0 ) 
   {
     MvdSetTgvVcal ( cline, ichan, tgvbit );
   }
   else if ( strcmp ( Cmd_vfb, arg2 ) == 0 ) 
   {
     MvdSetTgvVfb ( cline, ichan, tgvbit );
   }
   else if ( strcmp ( Cmd_vgate, arg2 ) == 0 ) 
   {
     MvdSetTgvVgate ( cline, ichan, tgvbit );
   }
   else if ( strcmp ( Cmd_vmid, arg2 ) == 0 ) 
   {
     MvdSetTgvVmid ( cline, ichan, tgvbit );
   }
   else if ( strcmp ( Cmd_vthr, arg2 ) == 0 ) 
   {
     MvdSetTgvVthr ( cline, ichan, tgvbit );
   }
   else if ( strcmp ( Cmd_calen, arg2 ) == 0 ) 
   {
     MvdSetTgvCalen ( cline, ichan, tgvbit );
   }
   else if ( strcmp ( Cmd_predis, arg2 ) == 0 ) 
   {
     MvdSetTgvPredis ( cline, ichan, tgvbit );
   }
   else if ( strcmp ( Cmd_dtwk, arg2 ) == 0 ) 
   {
     MvdSetTgvDtwk ( cline, ichan, tgvbit );
   }
   else if ( strcmp ( Cmd_csdis, arg2 ) == 0 ) 
   {
     MvdSetTgvCsdis ( cline, ichan, tgvbit );
   }
   else if ( strcmp ( Cmd_predisc, arg2 ) == 0 )
   {
     MvdSetTgvPredisC ( cline, ichan, tgvbit );
   }

   else
   {
      fprintf ( stderr, "MvdSetTgvBits: unknown command %s\n",arg2);
   }

   return;
}
