#include <stdlib.h>
#include <stdio.h>
void MvdShowTgvBits( int tgvbit[8][190]  )
{
   /* Show current TGV serial string setup                           */
   /* It makes three sets of tables, the first shows misc DAC values,*/
   /* the second shows the cal_enable and preamp_disable bits, the   */
   /* third shows the various bits related to the discriminator.     */

   /* input tgvbit                                                   */

   /* Author = JPSullivan  8-May-2001                                */

   /* Modified:                                                      */

   /* **********************************************************     */

   float MvdTgvDacVolts ( int tgvbit[8][190], int itgv, int bit1, 
                          int bitlast, float vmin, float vmax,
                          float dacmax );

   int i,j;

   /* for each of the 5 DACs, define a variable to hold its value in volts, */
   /* a pair of variables to define the LSB and MSB in the serial string,   */
   /* and a pair of variables to define the minimum and maximum values [V]. */

   float vthr;   /* Vthr (for discriminator) */
   int Vthr_bit1 = 165;
   int Vthr_last = 160;
   float Vthr_min = 1.86;    /* this corresponds to the lowest threshold */
   float Vthr_max =  0.00;   /* This corresponds to the highest threshold*/

   float vmid;   /* Vmid (not sure what it does)  */
   int Vmid_bit1 = 166;
   int Vmid_last = 171;
   float Vmid_min =  1.71;
   float Vmid_max =  4.50;

   float vgate;  /* Vgate -- controls Vthr sensitivity*/
   int Vgate_bit1 = 177;
   int Vgate_last = 172;
   float Vgate_min = 0.006;
   float Vgate_max = 3.68;

   float vfb;    /* vfb (preamp), higher number means slower return  */
                 /* to baseline */
   int Vfb_bit1 = 183;
   int Vfb_last = 178;
   float Vfb_min = 0.006;
   float Vfb_max =  5.02;

   float vcal;   /* vcal (for bench cal pulse) */
   int Vcal_bit1 = 189;
   int Vcal_last = 184;
   float Vcal_min =  3.48;
   float Vcal_max =  5.00;

   float dacmax = 63.;

   int ithr;

   /* ********************************************************** */

   /* header for table which follows: */
   printf ( " _____________________________________________________________________\n");
   printf ( " TGV serial string DAC values:\n");
   printf ( "  chip   Vthr   Vmid  Vgate    Vfb   Vcal   \n");
   printf ( "  num.    [V]    [V]    [V]    [V]    [V]   \n");
   printf ( " _____  _____  _____  _____  _____  _____   \n");

   /* **************************** */
   /* loop is over i=0,7 TGV chips */

   for (i=0; i<8; i++)
   {
     vthr = MvdTgvDacVolts ( tgvbit, i, Vthr_bit1, Vthr_last,
                             Vthr_min, Vthr_max, dacmax );

     vmid = MvdTgvDacVolts ( tgvbit, i, Vmid_bit1, Vmid_last,
                             Vmid_min, Vmid_max, dacmax );
 
     vgate= MvdTgvDacVolts ( tgvbit, i, Vgate_bit1, Vgate_last,
                             Vgate_min, Vgate_max, dacmax );
 
     vfb  = MvdTgvDacVolts ( tgvbit, i, Vfb_bit1, Vfb_last,
                             Vfb_min, Vfb_max, dacmax );
 
     vcal = MvdTgvDacVolts ( tgvbit, i, Vcal_bit1, Vcal_last,
                             Vcal_min, Vcal_max, dacmax );
 
     printf ( "  %4d  %5.2f  %5.2f  %5.2f  %5.2f  %5.2f \n",
              i, vthr, vmid, vgate, vfb, vcal );
   }

   /* ************************************************** */
   /* now the calibration enable and preamp disable bits */
   printf ( " _____________________________________________________________________\n");
   printf ( " TGV calibration enable and preamp disable bits:\n");
   printf ( "             calibration enable bits (1=on)  preamp disable bits (1=channel off) \n");
   printf ( "  chip             111111 11112222 22222233             111111 11112222 22222233 \n");        
   printf ( "  num.  01234567 89012345 67890123 45678901  01234567 89012345 67890123 45678901 \n");
   printf ( " _____  ___________________________________  ___________________________________ \n");

   /* **************************** */
   /* loop is over i=0,7 TGV chips */
   for (i=0; i<8; i++)
   {
     /* first print the "cal enable" bits -- 1 means channel will have a pulse sent */
     /* into it when a bench-cal enable mode bit is received  */

     printf ( "  %4d ", i );
     for ( j=0; j<=31; j++)
     {
        if ( j%8 == 0 ) printf ( " ");
        printf ("%1d", tgvbit[i][159-j] );
     }

     /* a space between the different sets of bits */
     printf (" ");

     /* now print the preamp disable bits  */

     for ( j=0; j<=31; j++)
     {
        if ( j%8 == 0 ) printf ( " ");
        printf ("%1d", tgvbit[i][96+j] );
     }

     printf ("\n");    /* end of line for this chip */
   }

   /* *************************************** */
   /* header for table of bits which follows: */
   printf ( " _____________________________________________________________________\n");
   printf ( " TGV discriminator setup:\n");
   printf ( "         discriminator disable bits (1=off)     discriminator tweak values (0-3) \n");
   printf ( "  chip             111111 11112222 22222233             111111 11112222 22222233 \n");        
   printf ( "  num.  01234567 89012345 67890123 45678901  01234567 89012345 67890123 45678901 \n");
   printf ( " _____  ___________________________________  ___________________________________ \n");

   /* **************************** */
   /* loop is over i=0,7 TGV chips */

   for (i=0; i<8; i++)
   {
     /* first print the "csdis" bits -- discriminator disable bit */

     printf ( "  %4d ", i );
     for ( j=0; j<=31; j++)
     {
        if ( j%8 == 0 ) printf ( " ");
        printf ("%1d", tgvbit[i][j] );
     }

     /* a space between the different sets of bits */
     printf (" ");

     /* now print the discriminator tweak bits (show the two */
     /* bits as a value from 0-3)                            */

     for ( j=0; j<=31; j++)
     {
        if ( j%8 == 0 ) printf ( " ");
        ithr = tgvbit[i][63-j] + 2*tgvbit[i][95-j];
        printf ("%1d", ithr );
     }

     printf ("\n");    /* end of line for this chip */

   }

   return;
}
float MvdTgvDacVolts ( int tgvbit[8][190], int itgv, int bit1, int bitlast,
                       float vmin, float vmax, float dacmax )
{
   /* Translate bit1 --> bitlast bits out of the serial string  */
   /* tgvbit for chip itgv into a value in volts. vmin and vmax */
   /* are the min and max values of the DAC in volts and dacmax */
   /* is the maximum value which can be specified by the DAC.   */

   /* input tgvbit = serial string bits for all 8 AMU/ADC chips      */
   /*       itgv   = chip number (1st index of tgvbit)               */
   /*       bit1   = 1st bit to convert (LSB)                        */
   /*       bitlast= last bit to convert (MSB)                       */
   /*       vmin   = voltage corresponds to DAC value = 0            */
   /*       vmax   = voltage value corresponding to DAC=dacmax       */
   /*       dacmax = max value the DAC can count to (always 63 here) */

   /* Author = JPSullivan  8-May-2001                                */

   /* Modified:                                                      */

   /* **********************************************************     */

   int j;        /* for loop over bits       */
   int ivolt;    /* DAC value, integer       */
   int ipower2;  /* to caclulate powers of 2 */
   float volt;   /* DAC value in volts       */

   /* ********************************************************** */

   ivolt = 0;
   ipower2=1;
   if ( bitlast>bit1 ) 
   {
   for (j=bit1; j<=bitlast; j++) 
     {
        ivolt = ivolt + tgvbit[itgv][j]*ipower2;
        ipower2*=2;
     }
   }
   else
   {
   for (j=bit1; j>=bitlast; j--) 
     {
        ivolt = ivolt + tgvbit[itgv][j]*ipower2;
        ipower2*=2;
     }
   }
   volt = ivolt/dacmax;
   volt *= (vmax-vmin);
   volt += vmin;

   return volt;
}
